/*
 * This is a plugin module for the dmx4linux drivers (http://llg.cubic.org/dmx4linux/).
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>
#include <signal.h>

#include <errno.h>

#include <sys/socket.h>
#include <netinet/in.h>

#include <dmx/plugin.h>


/*
 * some plugin-members
 */
static dmxuniverse_t *universes[20];
static int max_universes = sizeof(universes) / sizeof(universes[0]);

static int dmxin  = -1;
static int dmxout = -1;

static pthread_t rx_thread;
static pthread_t fader_thread;
static char      thread_active = 0;



/*
 * The thread that received incoming data from the dmx4linux-driver
 */
static void *dmx4linux_thread (void *arg)
{
  int i;
  unsigned char obuffer[20*512];
  unsigned char buffer[sizeof(obuffer)];

  printf ("dmx4linux_thread() pid=%d\n", getpid());
  fflush(stdout);

  if (dmxin==-1)
    return NULL;

  for (i=0; i<sizeof(buffer); i++)
    {
      obuffer[i] = 0;
      buffer[i]  = 0;
    }

  while (1)
    {
      struct sockaddr_in claddr;
      size_t             clsize = sizeof(claddr);
      int n;

      lseek(dmxin, 0, SEEK_SET);
      n = read (dmxin, buffer, sizeof(buffer));
      if (n>0)
        {
          for (i=0; i<max_universes && i < (n/512)+1; i++)
            {
              int k;
              char changed=0;
              for (k=0; k<512 && k<n; k++)
                {
                  if (obuffer[i*512+k] != buffer[i*512+k])
                    {
                      obuffer[i*512+k] = buffer[i*512+k];
                      changed=1;
                    }
                }
              if (changed && universes[i])
                {
                  int size = universes[i]->numslots;
                  memcpy (universes[i]->buffer, &buffer[i*512], (size>=512)?512:size);
                  universes[i]->notify(universes[i]);
                }
            }
        }
    }
  return NULL;
}

/*
 * allocated some needed resources.
 */
int initialize (void)
{
  memset (universes, 0, sizeof(universes));

  dmxout = open ("/dev/dmx", O_WRONLY);
  if (dmxout==-1)
    return -1;

  return 0;
}

/*
 * Is called before the plugin is unloaded and frees all used resources.
 */
int finalize ()
{
  int i;

  if (thread_active)
    {
      int stat = pthread_kill (rx_thread, SIGINT);
      if (!stat || stat == ESRCH)
        {
          void *ret = NULL;
          if (!pthread_join (rx_thread, &ret))
            {
              printf ("thread returned successfully and returned value %p\n", ret);
            }
        }
      thread_active = 0;
    }

  if (dmxin!=-1)
    close(dmxin);

  if (dmxout!=-1)
    close(dmxout);

  return 0;
}


/*
 * binds the universe object <u> to the plugins univere with the id <nr>.
 */
int bind_universe (int nr, dmxuniverse_t *u)
{
  if (nr>=0 && nr<max_universes && !universes[nr])
    {

      if (dmxin==-1)
        dmxin  = open ("/dev/dmxin", O_RDONLY);

      if (dmxin==-1)
        return -1;

      if (!thread_active)
        {
          if (pthread_create (&rx_thread, NULL, dmx4linux_thread, NULL)!=0)
            return -1;
          thread_active=1;
        }

      universes[nr] = u;
      return 0;
    }
  return -1;
}

/*
 * removes a binding between the universe object <u> and the
 * plugin's universe with the id <nr>.
 */
int unbind_universe (int nr, dmxuniverse_t *u)
{
  if (nr>=0 && nr<max_universes && universes[nr] && universes[nr]==u)
    {
      universes[nr] = NULL;
      return 0;
    }
  return -1;
}


/*
 * Sends out data stored in the universe object <u> to the universe eith the id <nr>.
 */
int send_universe (int nr, dmxuniverse_t * u)
{

  if (dmxout!=-1 && nr>=0 && nr<max_universes /* && universes[nr] && universes[nr]==u */)
    {
      lseek (dmxout, nr*512, SEEK_SET);
      write (dmxout, u->buffer, (u->numslots>512)?512:u->numslots);
      return 0;
    }
  return -1;
}


/*
 * returns some information about the plugin.
 */
int get_plugin_info (dmxplugin_info_t *info)
{
  if (info)
    {
      strcpy (info->vendor_name, "LLG");
      strcpy (info->proto_name, "dmx4linux");
      info->proto_release = 2.0;

      info->release = 1.0;
      strcpy (info->author, "Michael Stickel");
      strcpy (info->email,  "michael@cubic.org");
      info->num_universes = max_universes;
      return 0;
    }
  return -1;
}

/*-EOF-*/
