
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include <errno.h>

#include "dmxoe_codec.h"
#include "dmxoe.hh"


char DMXoESocket::IsConnected () {return connstat; };

DMXoESocket::~DMXoESocket ()
{
  if (sockfd)
    {
      connstat = 0;
      close (sockfd);
      shutdown(sockfd, 3);
    }
  if (valueBuffer)
    delete [] valueBuffer;
}

DMXoESocket::DMXoESocket (char *hname, int newSlots)
{
  Init (newSlots, hname);
  Connect();
}

DMXoESocket::DMXoESocket (int newSlots)
{
  Init (newSlots);
  Connect ();
}


int DMXoESocket::Init (size_t newSlots, char *newHname, unsigned short port)
{
  connstat = 1;
  sockfd = -1;
  maxSlots = newSlots;
  valueBuffer = NULL;
  netbuff = NULL;
  netbuffSize = 0x5000;

  hostname = newHname;
  if (!hostname) hostname="localhost";
  hostport = port;


  if (maxSlots<0 || maxSlots>512) maxSlots=512;
  valueBuffer = new unsigned char [maxSlots];
  if (!valueBuffer)
    {
      maxSlots=0;
      return -1;
    }

  netbuff = new unsigned char [netbuffSize];
  return 0;
}


int DMXoESocket::Connect ()
{
  struct sockaddr_in local;
  int  sock = -1;
  struct hostent * hent = NULL;
  int one = 1;
  int *oneptr = &one;

  if (sockfd!=-1) return 0;

  sock = socket(AF_INET, SOCK_DGRAM, 0);
  if(sock < 0)
    {
      perror("socket error");
      return -1;
    }
  sockfd = sock;

  if (setsockopt (sock, SOL_SOCKET, SO_BROADCAST, oneptr, sizeof(one)) == -1)
    {
#if 0
      fprintf (stderr, "Error getting broadcast permission: %s\n", stderror(errno));
#endif
      return -1;
    }


  hent = gethostbyname (hostname);
  bzero(&local, sizeof(local));
  local.sin_family = AF_INET;
  memcpy (&(local.sin_addr.s_addr), hent->h_addr_list[0], hent->h_length);
  local.sin_port = htons(hostport);
  {
    unsigned long ip = ntohl(local.sin_addr.s_addr);
    printf ("IP: %d.%d.%d.%d\n", (int)((ip>>24)&0xff),
                                 (int)((ip>>16)&0xff),
                                 (int)((ip>>8)&0xff),
                                 (int)(ip&0xff)  );
  }

  if(connect (sockfd, (struct sockaddr*)&local, sizeof(local)) < 0)
    {
      perror("connect error");
      return -1;
    }
  connstat = 1;
  return 0;
}


int DMXoESocket::flush (int size, unsigned char *buffer)
{
  return send_dmx (buffer, size);
}


int DMXoESocket::flush ()
{
  return flush (maxSlots, valueBuffer);
}


int DMXoESocket::send_dmx (unsigned char *dmxbuf,  size_t numslots)
{
  size_t   n = 0;

  n = shownet_encode (dmxbuf, numslots, netbuff, netbuffSize);
  if (n<0)
    {
      printf ("shownet_encode: netbuffer is too short\n");
      return -1;
    }
  return send (sockfd, netbuff, n, 0);
}


int DMXoESocket::setSlotValue (int slot, int value)
{
  if (slot < 0 || slot >= maxSlots)
    return -1;
  valueBuffer[slot] = value;
  return 0;
}

int DMXoESocket::getSlotValue (int slot, int value)
{
  if (slot < 0 || slot >= maxSlots)
    return -1;
  return valueBuffer[slot];
}


