/*
 * trycatch.h
 *
 * some exception handling in plain C.
 *
 * author: Michael Stickel <michael@cubic.org>
 * date:   12.11.2000
 */
#ifndef __TRYCATCH_H__
#define __TRYCATCH_H__

#include <setjmp.h>
#include <signal.h>

typedef struct
{
 void (*sig_SIGSEGV) (int);
 void (*sig_SIGFPE)  (int);
#if 0
 void (*sig_SIGPIPE) (int);
#endif
 jmp_buf env;
} trycatch_stackelem_t;

typedef int cause_t;


extern int   env_sp;
extern trycatch_stackelem_t __trycatchstack[100];

#define __UNWIND_SIG(signame)  signal(signame,__trycatchstack[env_sp-1].sig_##signame)
#define __UNWIND_TRY           (__UNWIND_SIG(SIGSEGV),__UNWIND_SIG(SIGFPE),--env_sp)
#define __TRY_SAVESIG(signame) (__trycatchstack[env_sp-1].sig_##signame=signal(signame,compute_sigv))
#define __TRY_SAVESIGS         (__TRY_SAVESIG(SIGSEGV),__TRY_SAVESIG(SIGFPE))
#define __TRY_SAVEENV          (sigsetjmp(__trycatchstack[env_sp-1].env,1))


#define TRY            env_sp++;if(__TRY_SAVEENV==0){ __TRY_SAVESIGS;{
#define TRYwith(cause) env_sp++;cause=__TRY_SAVEENV;if(cause==0){__TRY_SAVESIGS;{
#define CATCH                                                                      }__UNWIND_TRY;}else

#define catchreturn(rv)   {__UNWIND_TRY; return rv;}
#define catchcontinue     {__UNWIND_TRY; continue;}


extern void compute_sigv (int sig);

#endif
